local MAX_LIGHT_ARMOR = 25
local MIN_HEAVY_ARMOR = -40
local MAX_UNARMORED = 40


--percentage of armor part contributing to evasion


armorParts = {
	[0] = 0.1,
	[1] = 0.4,
	[2] = 0.05,
	[3] = 0.05,
	[4] = 0.15,
	[5] = 0.15,
	[6] = 0.05,
	[7] = 0.05,
--	[8] = 0
--	[9] = 0.05,
--	[10] = 0.05
}


local function calcArmorEvasion(reference)
	local evasion = 0
	local mobile = reference.mobile
	local unarmored = 0
	local light = 0
	local heavy = 0
	if mobile == nil then -- check for disabled actors
		return 
	end
	for i, value in pairs(armorParts) do
		local stack = tes3.getEquippedItem{actor = reference, objectType = tes3.objectType.armor, slot = i}
		if i == 6 or i == 7 then	-- if no gloves - check for bracers
			if not stack then stack = tes3.getEquippedItem{actor = reference, objectType = tes3.objectType.armor, slot = i+3} end
		end
		if stack then
			item = stack.object
			if item.weightClass == 0 then
				light = light + value
			elseif item.weightClass == 2 and  mobile:getSkillValue(3) < 100 then
				heavy = heavy + value
			end
		else
			unarmored = unarmored + value
		end
	end
	evasion = math.floor(light * MAX_LIGHT_ARMOR * mobile:getSkillValue(21)*0.01 + unarmored * MAX_UNARMORED * mobile:getSkillValue(17)*0.01 + heavy * MIN_HEAVY_ARMOR * (100 - mobile:getSkillValue(3))*0.01)
	if reference.data.armorEvasion then	-- removing previous value
		mobile.sanctuary = mobile.sanctuary - reference.data.armorEvasion
	end
	mobile.sanctuary = mobile.sanctuary + evasion
	reference.data.armorEvasion = evasion -- remembering new value
end


local function onEquipped(e)
	if e.item.objectType ~= tes3.objectType.armor then
		return
	end
	calcArmorEvasion(e.reference)
end


local function onUnequipped(e)
	if e.item.objectType ~= tes3.objectType.armor then
		return
	end
	calcArmorEvasion(e.reference)
end


-- initial calculation for npcs
local function onMobileActivated(e)
	if e.reference.baseObject.objectType == tes3.objectType.npc then
		calcArmorEvasion(e.reference)
	end
end


-- initial calculation for pc
local function onUiRefreshed(e)
	calcArmorEvasion(tes3.player)
	local gmst = tes3.findGMST("fUnarmoredBase1")
	gmst.value = 0
end


local function onSkillRaised(e)
	if skill ~= 21 and skill ~= 3 and skill ~= 17 then return end
	calcArmorEvasion(tes3.player)
end


-- calculates evasion every frame for actors under armor skills modifing effects
local function onSpellTick(e)
	if e.effectId ~= 83 and e.effectId ~= 89 and e.effectId ~= 21 and e.effectId ~= 26 and e.effectId ~= 78 then
		return
	end
	local skill = e.source.effects[e.effectIndex + 1].skill
	if skill ~= 3 and skill ~= 21 and skill ~= 17 then return end
	local actor = e.target
	if not actor then actor = e.caster end
	timer.start{
		duration = 0.1, 
		callback = function()
			calcArmorEvasion(actor)
			if e.effectId == 89 then -- absorb skill
				calcArmorEvasion(e.caster)
			end
		end
	}	
end


local function onAttack(e)
	if e.targetReference ~= tes3.player then return end
	local light = 0
	local unarmored = 0
	for i, value in pairs(armorParts) do
		local stack = tes3.getEquippedItem{actor = tes3.player, objectType = tes3.objectType.armor, slot = i}
		if i == 6 or i == 7 then
			if not stack then stack = tes3.getEquippedItem{actor = reference, objectType = tes3.objectType.armor, slot = i+3} end
		end
		if stack and stack.object.weightClass == 0 then
			light = light + value
		elseif not stack then
			unarmored = unarmored + value
		end
	end
	if unarmored > 0 then
		tes3.mobilePlayer:exerciseSkill(17, unarmored)
	end
	if light > 0 then
		tes3.mobilePlayer:exerciseSkill(21, light*0.5)
	end
	
end


local function initialized(e)
	event.register("equipped", onEquipped)
	event.register("unequipped", onUnequipped)
	event.register("uiRefreshed", onUiRefreshed)
	event.register("mobileActivated", onMobileActivated)
	event.register("skillRaised", onSkillRaised)
	event.register("spellTick", onSpellTick)
	event.register("attack", onAttack)
end

event.register("initialized", initialized)